#                © Copyright 2023
#         Licensed under the MIT License
#       https://opensource.org/licenses/MIT
#    https://github.com/Den4ikSuperOstryyPer4ik

from gdzru.types.book import BookInfo
from gdzru.types.full_book_list import FullBookList
from gdzru.types.task import TaskInfo

__all__ = [
    "BookInfo",
    "FullBookList",
    "TaskInfo",
]
